#include <iostream>

#include "HandEye.h"
#include "RVC/RVC.h"

#define ERROR_EXIT()                                                                                                                       \
    std::cout << "error code:" << RVC::GetLastError() << "\n";                                                                             \
    return -1;

int main(int argc, char **argv) {
    // RVC Capture process
    RVC::SystemInit();

    RVC::Device d[10];
    size_t actualSize;
    RVC::SystemListDevices(d, 10, &actualSize, RVC::SystemListDeviceType::USB);

    if (actualSize == 0) {
        return -1;
    }
    auto device = d[0];
    auto x1 = RVC::X1::Create(device);
    if (!x1.IsValid()) {
        ERROR_EXIT();
    }
    if (!x1.Open()) {
        ERROR_EXIT();
    }
    if (!x1.Capture()) {
        ERROR_EXIT();
    }
    auto pc = x1.GetPointMap();
    auto rimg = x1.GetImage();
    // data process
    // process image2D
    RVC::ImageType::Enum imageType = rimg.GetType();
    int imageRows = rimg.GetSize().rows, imageCols = rimg.GetSize().cols;
    int imageSize = imageRows * imageCols;
    int byteSize;
    int cvType;
    switch (imageType) {
    case RVC::ImageType::Enum::Mono8:
        cvType = CV_8UC1;
        break;
    case RVC::ImageType::Enum::RGB8:
        cvType = CV_8UC3;
        break;
    case RVC::ImageType::Enum::BGR8:
        cvType = CV_8UC3;
        break;
    default:
        return -1;
    }
    cv::Mat image2D_0(imageRows, imageCols, cvType);
    memcpy(image2D_0.data, rimg.GetDataConstPtr(), imageSize * CV_ELEM_SIZE(cvType));
    // process points
    cv::Mat point_0(imageRows, imageCols, CV_64FC3);
    memcpy(point_0.data, pc.GetPointDataConstPtr(), imageSize * CV_ELEM_SIZE(CV_64FC3));
    // RVCHandEye
    RVCHandEye::MarkerBoardResult markerRes;
    auto ret = RVCHandEye::SolveMarkerBoardResults(image2D_0, point_0, markerRes);
    if (ret != RVCHandEye::Err_OK) {
        return -1;
    }
    RVCHandEye::FinalResult finalRes;
    ret = RVCHandEye::KukaSolveFinalResult({markerRes}, {{50.1, 41.12, 30.2, 0.15, 0.11, 0.55}},
        RVCHandEye::AlgoParam{RVCHandEye::EYE_TO_HAND, true, true, false, 0.5}, finalRes);
    if (ret != RVCHandEye::Err_OK) {
        return -1;
    } else {
    }

    RVC::SystemShutdown();
    return 0;
}