import PyRVC as RVC
import PyHandEye as he
import numpy as np

## get data
# Initialize RVC X system.
RVC.SystemInit()

# Choose RVC X Camera type (USB, GigE or All)
opt = RVC.SystemListDeviceTypeEnum.All

# Scan all RVC X USB Camera devices.
ret, devices = RVC.SystemListDevices(opt)
print("RVC X USB Camera devices number:%d" % len(devices))

# Find whether any RVC X Camera is connected or not.
if len(devices) == 0:
    print("Can not find any RVC X USB Camera!")
    RVC.SystemShutdown()
    exit()
print("devices size = %d" % len(devices))

# Create a RVC X Camera and choose use left side camera.
x = RVC.X1.Create(devices[0], RVC.CameraID_Left)

# Test RVC X Camera is valid or not.
if x.IsValid() == True:
    print("RVC X Camera is valid!")
else:
    print("RVC X Camera is not valid!")
    RVC.X1.Destroy(x)
    RVC.SystemShutdown()
    exit()

# Open RVC X Camera.
ret1 = x.Open()

# Test RVC X Camera is opened or not.
if ret1 and x.IsOpen() == True:
    print("RVC X Camera is opened!")
else:
    print("RVC X Camera is not opened!")
    RVC.X1.Destroy(x)
    RVC.SystemShutdown()
    exit()

# Capture a point map and a image.
ret2 = x.Capture()

if ret2 == True:
    print("RVC X Camera capture successed!")
    img = x.GetImage()
    width = img.GetSize().cols
    height = img.GetSize().rows
    point = x.GetPointMap()
else:
    print("RVC X Camera capture successed!")
    RVC.X1.Destroy(x)
    RVC.SystemShutdown()
    exit()

## process
img_n = np.array(img, copy=False)
point_n = np.array(point, copy=False)

"""
@param image2D 2D图
@param imagePoint 点云数据
@param is_BGR 当输入图像为RGB或BGR时，第三个参数`is_BGR`生效。当输入图像为RGB时，该参数应该为False，否则应该为True， 默认值为True

也可以通过文件路径载入图像数据
markerRes, ret = he.SolveMarkerBoardResultsByPath(
    img2DPath=imgPath, pointPath=pointPath
)
"""
markerRes, ret = he.SolveMarkerBoardResults(
    image2D=img_n, imagePoint=point_n, is_BGR=True
)

if ret != he.ErrorCode.Err_OK:
    print("errorCode:", ret)
    exit()

algoParam = he.AlgoParam()
algoParam.type = he.HandEyeType.EYE_TO_HAND  # 眼在手外
algoParam.is_m = False  # 单位为m
algoParam.is_radian = False  # 机器人姿态为弧度制
algoParam.exhaust = True  # 暴力求解
algoParam.ransac_dist = 0.001  # 最大误差
"""
@param markerResults 标定板坐标计算结果列表，列表中每一项代表标定板坐标计算结果
@param robotParams 机器人参数列表，列表中每一项代表机器人参数，机器人参数前三项为坐标值，后三(四)项为机器人姿态，单位在algoParam中进行设置.
@param algoParam 算法参数
"""
finalRes, ret = he.KukaSolveFinalResult(
    markerResults=[
        markerRes,  # [markerRes], 每个SolveMarkerBoardResults的结果作为列表元素
    ],
    robotParams=[
        [387.03, 135.89, 77.20, 55.36, -0.18, 163.17],
        # [410.39, 129.88, 713.40, 31.76, -9.63, 172.95],
        # [400.41, 153.52, 738.25, 20.39, -16.81, 179.09],
        # [384.26, 182.34, 782.96, 13.52, -24.56, -173.12],
        # [358.42, 210.22, 829.32, 6.95, -30.87, -161.99],
        # [426.30, 65.69, 621.88, -3.92, -5.78, -174.43],
        # [453.37, 10.60, 698.40, -24.65, -18.29, -163.53],
    ],
    algoParam=algoParam,
)
if ret != he.ErrorCode.Err_OK:
    print("errorCode:", ret)
    exit()
